<?php

namespace Bixo\Search\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Search\Http\Requests\SearchesRequest;
use Bixo\Search\Interfaces\SearchesRepositoryInterface;
use Bixo\Search\Models\Searches;

/**
 * Resource controller class for searches.
 */
class SearchesResourceController extends BaseController
{

    /**
     * Initialize searches resource controller.
     *
     * @param type SearchesRepositoryInterface $searches
     *
     * @return null
     */
    public function __construct(SearchesRepositoryInterface $searches)
    {
        parent::__construct();
        $this->repository = $searches;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Search\Repositories\Criteria\SearchesResourceCriteria::class);
    }

    /**
     * Display a list of searches.
     *
     * @return Response
     */
    public function index(SearchesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Search\Repositories\Presenter\SearchesPresenter::class)
                ->$function();
        }

        $searches = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('search::searches.names'))
            ->view('search::searches.index', true)
            ->data(compact('searches', 'view'))
            ->output();
    }

    /**
     * Display searches.
     *
     * @param Request $request
     * @param Model   $searches
     *
     * @return Response
     */
    public function show(SearchesRequest $request, Searches $searches)
    {

        if ($searches->exists) {
            $view = 'search::searches.show';
        } else {
            $view = 'search::searches.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('search::searches.name'))
            ->data(compact('searches'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new searches.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SearchesRequest $request)
    {

        $searches = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('search::searches.name')) 
            ->view('search::searches.create', true) 
            ->data(compact('searches'))
            ->output();
    }

    /**
     * Create new searches.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SearchesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $searches                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('search::searches.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('search/searches/' . $searches->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/search/searches'))
                ->redirect();
        }

    }

    /**
     * Show searches for editing.
     *
     * @param Request $request
     * @param Model   $searches
     *
     * @return Response
     */
    public function edit(SearchesRequest $request, Searches $searches)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('search::searches.name'))
            ->view('search::searches.edit', true)
            ->data(compact('searches'))
            ->output();
    }

    /**
     * Update the searches.
     *
     * @param Request $request
     * @param Model   $searches
     *
     * @return Response
     */
    public function update(SearchesRequest $request, Searches $searches)
    {
        try {
            $attributes = $request->all();

            $searches->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('search::searches.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('search/searches/' . $searches->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('search/searches/' . $searches->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the searches.
     *
     * @param Model   $searches
     *
     * @return Response
     */
    public function destroy(SearchesRequest $request, Searches $searches)
    {
        try {

            $searches->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('search::searches.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('search/searches/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('search/searches/' . $searches->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple searches.
     *
     * @param Model   $searches
     *
     * @return Response
     */
    public function delete(SearchesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('search::searches.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('search/searches'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/search/searches'))
                ->redirect();
        }

    }

    /**
     * Restore deleted searches.
     *
     * @param Model   $searches
     *
     * @return Response
     */
    public function restore(SearchesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('search::searches.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/search/searches'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/search/searches/'))
                ->redirect();
        }

    }

}
